package net.wanji.com.service.controller.impl;

import cn.hutool.core.util.HexUtil;
import net.wanji.com.cache.CrossInfoCache;
import net.wanji.com.common.constants.Constants;
import net.wanji.com.netty.NettyClient;
import net.wanji.com.pojo.dtconvert.FaultInfoPojo;
import net.wanji.com.pojo.dtconvert.RunningStatusPojo;
import net.wanji.com.pojo.netty.MessageResultPojo;
import net.wanji.com.service.controller.SignalStatusService;
import net.wanji.common.enums.BaseEnum;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/05/08 17:28
 */
@Service
public class DTSignalStatusServiceImpl implements SignalStatusService {

    @Resource
    private CrossInfoCache crossInfoCache;

    @Override
    public String getBeanMark() {
        return BaseEnum.VendorTypeEnum.DT.getNick();
    }

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm() {
        return null;
    }

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) {
        List<FaultInfoPojo> faultInfoPojos = getFaultInfoPojos(crossId);
        List<String> lightsStatus = getLightsStausInfoPojos(crossId);
        List<RunningStatusPojo> runningStatusPojos = getRunningStatusPojos(crossId);
        return null;
    }

    private List<RunningStatusPojo> getRunningStatusPojos(String crossId) {
        List<RunningStatusPojo> runningStatusPojos = new ArrayList<>();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_RUNNING_STATUS);
        String runningStatusSign = Constants.COMMAND_RUNNING_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String runnningStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), runnningStatusHex, runningStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int size = body.length() / 14;
            List<String> paramList = new ArrayList<>();
            for (int i = 0; i < size; i++) {
                String param = body.substring(i * 14 + 6, i * 14 + 10);
                if (!StringUtils.equalsIgnoreCase("0000", param)) {
                    paramList.add(param);
                }
            }
            for (int i = 0; i < paramList.size() / 4; i++) {
                RunningStatusPojo runningStatusPojo = new RunningStatusPojo();
                runningStatusPojo.setControlCrossId(HexUtil.hexToInt(paramList.get(i * 4)));
                runningStatusPojo.setControlMode(paramList.get(i * 4 + 1));
                runningStatusPojo.setSchemeId(HexUtil.hexToInt(paramList.get(i * 4 + 2)));
                runningStatusPojo.setPhaseStageId(HexUtil.hexToInt(paramList.get(i * 4 + 3)));
                runningStatusPojos.add(runningStatusPojo);
            }
        }
        return runningStatusPojos;
    }

    private List<String> getLightsStausInfoPojos(String crossId) {
        List<String> lightsStatusList = new ArrayList<>();
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_LIGHT_STATUS);
        String lightStatusSign = Constants.COMMAND_LIGHT_STATUS.concat(Constants.COMMAND_QUERY_REPLY);
        String lightStatusHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), lightStatusHex, lightStatusSign, 300);
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int lightsStatusSize = body.length() / 14;
            for (int i = 0; i < lightsStatusSize; i++) {
                String param = body.substring(i * 14 + 8, i * 14 +10);
                if (!StringUtils.equalsIgnoreCase("00", param)) {
                    lightsStatusList.add(param);
                }
            }
        }
        return lightsStatusList;
    }

    public static void main(String[] args) {
        String message = String.format("010001000000010001%s010105%s020000", Constants.COMMAND_QUERY, Constants.COMMAND_FAULT_LIST);
        String faultHex = Constants.buildMessage(message);
        System.err.println(faultHex);
    }

    private List<FaultInfoPojo> getFaultInfoPojos(String crossId) {
        CrossInfoPO crossInfoPO = crossInfoCache.getCrossInfoCache().get(crossId);
        String message = String.format(Constants.COMMAND_COMMON, Constants.COMMAND_QUERY, Constants.COMMAND_FAULT_LIST);
        String faultSign = Constants.COMMAND_FAULT_LIST.concat(Constants.COMMAND_QUERY_REPLY);
        String faultHex = Constants.buildMessage(message);
        MessageResultPojo resultPojo = NettyClient.sendMessage(crossInfoPO.getIp(), crossInfoPO.getPort(), faultHex, faultSign, 300);
        List<FaultInfoPojo> faultInfoPojos = new ArrayList<>();
        if (Objects.nonNull(resultPojo) && StringUtils.isNotBlank(resultPojo.getHexMessageResult())) {
            String hexMessageResult = resultPojo.getHexMessageResult();
            String body = hexMessageResult.substring(32, hexMessageResult.length() - 2);
            int lightsStatusSize = body.length() / 14 * 64;
            int paramIndex = 8;
            for (int i = 0; i < lightsStatusSize; i++) {
                String param = body.substring(paramIndex * i, paramIndex * i + 8);


            }

        }
        return null;
    }

    @Override
    public List<LightsStatusVO> lightStatus() {
        return null;
    }

    @Override
    public List<LightsStatusVO> lightStatus(String crossId) {
        return null;
    }
}
